----------------------------------------------------------------------------------
-- Company: 
-- Engineer: 
-- 
-- Create Date:    17:40:25 01/30/2015 
-- Design Name: 
-- Module Name:    PipeControl - Behavioral 
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: 
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.std_logic_arith.all;
-- use ieee.numeric_std.all;

-- Uncomment the following library declaration if using
-- arithmetic functions with Signed or Unsigned values
--use IEEE.NUMERIC_STD.ALL;

-- Uncomment the following library declaration if instantiating
-- any Xilinx primitives in this code.
--library UNISIM;
--use UNISIM.VComponents.all;

entity PipeControl is
	port (
	   CLK: IN STD_LOGIC;                          -- High speed clock
		EN: IN STD_LOGIC;                           -- Enable signal for slow counting
		RESET: IN STD_LOGIC;                        -- Reset signal
		Dual: IN STD_LOGIC;                         -- 2 Boards connected
		MaxCount: IN STD_LOGIC_VECTOR (8 downto 0); -- "111111111" -> 31 Bit
		Code: OUT STD_LOGIC_VECTOR (24 downto 0);   -- Output full code
		VALID: OUT STD_LOGIC;                       -- Output Code is ok
      JA: OUT STD_LOGIC_VECTOR (5 downto 0);      -- Output CLK2
		DIN: IN STD_LOGIC_VECTOR (1 downto 0)       -- Signal input single ended
   );
end PipeControl;

architecture Behavioral of PipeControl is

  SIGNAL      cntDiv: std_logic_vector(8 downto 0); -- Count to 512
  SIGNAL      intJA: std_logic_vector(5 downto 0); -- Count to 512
  SIGNAL      cALL: std_logic_vector(32 downto 0); -- Count to 512
  Signal      validx: std_logic;
begin
  -- Generate divided clock for multiplexing LED
  ckcnt: process(clk,en,reset,call)
  begin
    if reset = '1' then
	    cntDiv <= "000000000";
		 intJA <="101010";
		 cALL <= "000000000000000000000000000000000";
		 Code <= cALL(30 downto 6); 
	 elsif clk'event and clk='1' and en='1' then 
	      if (cntDiv < MaxCount) then   -- count to 512 512/8= 64 cycles
		                                 -- 2 cycles one sample external
													-- 2 cycles one bit sample -> 31 bit  
		    cntDiv <= (unsigned(cntDiv) + 1);
 	      else                           -- Transfer data to register reset cntDiv
		    cntDiv <= (others => '0'); 			 
		   
			if (Dual ='0') then
		     if (MaxCount = "111111111") then -- sample 31 bits output 24
	          Code <= cALL(30 downto 6); 
		     elsif (MaxCount = "011111111") then -- sample 15 bits 
             Code <= cALL(14 downto 0) & "0000000000"; 
		     elsif ( MaxCount = "001111111") Then -- sample 7 bits
             Code <= cALL(6 downto 0)  & "000000000000000000"; 
		     elsif ( MaxCount = "000111111") then -- sample 3 bits
             Code <= cALL(2 downto 0)  & "0000000000000000000000";
		     elsif ( MaxCount = "000011111") then -- sample 1 bits
             Code <= cALL(0)  & "000000000000000000000000";
           else
  		       Code <= "0000000000000000000000000";
           end if;
          else  -- Dual boards and dual strobe
		     if (MaxCount = "011111111") then -- sample 31 bits output 24
	          Code <= cALL(30 downto 6); 
		     elsif (MaxCount = "001111111") then -- sample 15 bits 
            Code <= cALL(14 downto 0) & "0000000000"; 
		     elsif ( MaxCount = "000111111") Then -- sample 7 bits
            Code <= cALL(6 downto 0)  & "000000000000000000"; 
		     elsif ( MaxCount = "000011111") then -- sample 3 bits
            Code <= cALL(2 downto 0)  & "0000000000000000000000";
		     elsif ( MaxCount = "000001111") then -- sample 1 bits
            Code <= cALL(0)  & "000000000000000000000000";
           else
  		      Code <= "0000000000000000000000000";
           end if;
		    end if; -- end Dual boards and dual strobe
		   end if;
			if (cntDiv = "000000000") then
		    cALL <= "000000000000000000000000000000000";
 	      end if;
     
		   if (cntDiv(2 downto 0)="000") then
		    intJA <="101010";                   -- all CLK TGs open every 8th cycle
		   elsif (cntDiv(8 downto 4)="00000") then
          intJA <="011010";                   -- CLK1 TG closed sample Vin
		   elsif (cntDiv(3)='0') then
          intJA <="100110";                   -- CLK2 TG closed send signal to folder
		   elsif (cntDiv(3)='1') then
          intJA <="101001";                   -- CLK3 TG closed send residue to input capacitance
         end if;		
		
		   if (cntDiv(3 downto 0)="0110") then
		     cALL <= call(31 downto 0) & DIN(0);   -- sample data and put it into shift register
         end if;		
		   if (cntDiv(3 downto 0)="1110") and (Dual='1') then
		     cALL <= call(31 downto 0) & DIN(1);   -- sample data and put it into shift register
         end if;
      
	 end if;
  end process;

  validP: process(clk,en,reset,call)
  begin
    if reset = '1' then
		 validx <='0';
	 elsif clk'event and clk='1' then 
	      if ((cntDiv = "000000000") and (en='1')  ) then validx <='1';
			else validx <='0';
			end if;
    end if;
  end process;
	 
    valid <= validx;
	 JA <= intJA;

end Behavioral;

