----------------------------------------------------------------------------------
-- Company: 
-- Engineer: 
-- 
-- Create Date:    13:27:42 06/14/2011 
-- Design Name: 
-- Module Name:    Basys_Gnome - Behavioral 
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: 
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.std_logic_arith.all;

-- Uncomment the following library declaration if using
-- arithmetic functions with Signed or Unsigned values
--use IEEE.NUMERIC_STD.ALL;

-- Uncomment the following library declaration if instantiating
-- any Xilinx primitives in this code.
library UNISIM;
use UNISIM.VComponents.all;

entity Pipe_NEXYS3 is
   generic(
      WBUFFER: natural := 14 -- Buffer depth X16  14 -> 16k ok
   );
	port (
		CLK1: IN STD_LOGIC;				            -- clock (naturally)
   	CLK2: IN STD_LOGIC;				            -- clock (naturally) not used
		btn: IN STD_LOGIC_VECTOR (3 DOWNTO 0);	   -- Buttons on Basys
		seg: OUT STD_LOGIC_VECTOR (6 DOWNTO 0);	-- Segments
		an: OUT STD_LOGIC_VECTOR (3 DOWNTO 0);	   -- Digit
		dp: OUT STD_LOGIC;				               -- dot in Segment
		led: OUT STD_LOGIC_VECTOR (7 DOWNTO 0);	-- Segments
      sw: IN STD_LOGIC_VECTOR (7 DOWNTO 0);
      JA: OUT STD_LOGIC_VECTOR (5 downto 0);       -- Output signals for TG2 CLK1, CLK2, CLK3
		DIN: IN STD_LOGIC_VECTOR (1 downto 0);       -- Signal input single ended
		JB: OUT STD_LOGIC_VECTOR (3 downto 0);        --    NEXYS3 
		TxD: OUT STD_LOGIC;        --    NEXYS3 
		RxD: IN STD_LOGIC          --    NEXYS3 
	);
end Pipe_NEXYS3;

-- switch (sw4) = 1 controls static TGs or dynamic TGs

-- switch (3 downto 0) control pulsewidth and sample rate
-- 0000 5400ms
-- 0001  340ms
-- 0010   84ms  (1.32ms pw 758Hz)  1.32ms*64 = 91ms is fs= 11Hz for 32 Bit  -> 32 times value UART sent
-- 0011   22ms -> fs= 46Hz  32 Bit  -> 8 times UART sent
-- 0100   11ms  (164us pw)
-- 0101    5ms  (80us pw)
-- 0110  2.5ms  (40us pw)
-- 0111  1.25ms (20us pw 50kHz)
-- 1000  625us
-- 1001  312us
-- 1010  156us
-- 1011  78us

-- switch (5 downto 6) control number of bits
-- 00  32
-- 01  16
-- 10   8
-- 11   2

-- switch (7) dual data strobe for 2 boards connected

-- btn(0) B8 reset ->center button
-- btn(1) activate average -> show in dp BTNU A8

-- each new value generates a one clock pulse valid signal


architecture Behavioral of Pipe_NEXYS3 is
 
 -- Component Declaration
 	COMPONENT PipeControl
	PORT(
		CLK : IN std_logic;
		EN : IN std_logic;
		RESET : IN std_logic;
		Dual: IN STD_LOGIC;                           -- 2 Boards connected
		MaxCount: IN STD_LOGIC_VECTOR (8 downto 0);   -- "111111111" -> 31 Bit
		DIN : IN std_logic_VECTOR (1 downto 0);          
		Code : OUT std_logic_vector(24 downto 0);
		VALID : OUT std_logic;
		JA : OUT std_logic_vector(5 downto 0)
		);
	END COMPONENT;

-- component blk_mem_gen_v4_3
--	port (
--	clka: in std_logic;
--	wea: in std_logic_vector(0 downto 0);
--	addra: in std_logic_vector(13 downto 0);
--	dina: in std_logic_vector(11 downto 0);
--	douta: out std_logic_vector(11 downto 0));
-- end component;


 -- Component Declaration
    COMPONENT SegCntrl
      PORT(
		            CLK1: IN STD_LOGIC;				            -- clock (naturally)
                Digit0: IN STD_LOGIC_VECTOR (3 DOWNTO 0);
		          Digit1: IN STD_LOGIC_VECTOR (3 DOWNTO 0);
		          Digit2: IN STD_LOGIC_VECTOR (3 DOWNTO 0);
		          Digit3: IN STD_LOGIC_VECTOR (3 DOWNTO 0);
		           reset: IN STD_LOGIC;
		             seg: OUT STD_LOGIC_VECTOR (6 DOWNTO 0);	-- Segments
		              an: OUT STD_LOGIC_VECTOR (3 DOWNTO 0);	   -- Digit
		              dp: OUT STD_LOGIC				               -- dot in Segment
                  );
    END COMPONENT;


    COMPONENT CLK_DIV 
    Port ( CLK : in  STD_LOGIC;
	        reset : in STD_LOGIC;
           EN_4 : out  STD_LOGIC;
           EN_16 : out  STD_LOGIC;
           EN_32 : out  STD_LOGIC;
           EN_64 : out  STD_LOGIC;
           EN_128 : out  STD_LOGIC;
           EN_256 : out  STD_LOGIC;
           EN_512 : out  STD_LOGIC;
           EN_1024 : out  STD_LOGIC;
           EN_2048 : out  STD_LOGIC;
           EN_4096 : out  STD_LOGIC;
           EN_16K : out  STD_LOGIC;
           EN_65K : out  STD_LOGIC;
			  EN_1M : out  STD_LOGIC);
    end COMPONENT;

   COMPONENT DPRAM
   generic(
      B: natural := 16; -- number of bits
      W: natural := 11 -- number of address bits
   );
	PORT(
		clk : IN std_logic;
		we : IN std_logic;
		w_data : IN std_logic_vector(B - 1 downto 0);
		w_addr : IN std_logic_vector(W - 1 downto 0);          
		r_addr : IN std_logic_vector(W - 1 downto 0);          
		r_data : OUT std_logic_vector(B - 1 downto 0)
		);
	END COMPONENT;

  COMPONENT uart
   port(
      clk, reset: in std_logic;
      rd_uart, wr_uart: in std_logic;
      rx: in std_logic;
      w_data: in std_logic_vector(7 downto 0);
      tx_full, tx_empty, rx_empty: out std_logic;
      r_data: out std_logic_vector(7 downto 0);
      tx: out std_logic
   );
   end COMPONENT;


SIGNAL BCode, TCode, Code, ACode: STD_LOGIC_VECTOR (24 downto 0);
SIGNAL countx, county: STD_LOGIC_VECTOR (17 downto 0);

SIGNAL D: STD_LOGIC;
SIGNAL DXA,DX0,DX1,DX2,DX3,DX4,DX5: STD_LOGIC_VECTOR (3 downto 0);
SIGNAL JAX: STD_LOGIC_VECTOR (5 downto 0);
SIGNAL reset: STD_LOGIC;
SIGNAL ENX, EN_4, EN_16, EN_32, EN_64, EN_128, EN_256, EN_4096, EN_16K, EN_65K, EN_1M: STD_LOGIC;
SIGNAL EN_512, EN_1024, EN_2048, EN_T: STD_LOGIC;
SIGNAL sel: STD_LOGIC_VECTOR (3 downto 0);

SIGNAL wr_uart,tx_full, rx_empty: STD_LOGIC;
SIGNAL w_data,RD_data: STD_LOGIC_VECTOR(7 downto 0);
SIGNAL i_data: STD_LOGIC_VECTOR(4 downto 0);
SIGNAL ST: STD_LOGIC_VECTOR(4 downto 0);
SIGNAL decode: STD_LOGIC_VECTOR(2 downto 0);
SIGNAL trigger: STD_LOGIC_VECTOR(1 downto 0);
SIGNAL Valid: STD_LOGIC;
SIGNAL TXI: STD_LOGIC;
SIGNAL MaxCount: STD_LOGIC_VECTOR(8 downto 0);
SIGNAL douta: STD_LOGIC_VECTOR(11 downto 0);
SIGNAL addra: STD_LOGIC_VECTOR(13 downto 0);
SIGNAL wea: STD_LOGIC_VECTOR(0 downto 0);
SIGNAL dpx:STD_LOGIC;
SIGNAL AVG:STD_LOGIC;

SIGNAL WE:STD_LOGIC;
SIGNAL start:STD_LOGIC;
SIGNAL tx_empty:STD_LOGIC;

------------------------------------------------------------------------------------------
begin

 reset<=btn(0);

 led <= Code(24 downto 17);
-- when button pressed Calibrate
-- Initialize RAM with 0000
-- start counter when BCode above #00000
-- whith WE save counts at address BCode
-- when maximum reached, fill rest RAM
-- Then use lookup RAM
  
-- RAM Memory
-- your_instance_name : blk_mem_gen_v4_3
--		port map (
--			clka => clk1,
--			wea => wea,
--			addra => BCode(24 downto 11),
--			dina => BCode(24 downto 13),   
--			douta => douta);

-- CLK Divider

 CLKDIV0: CLK_DIV port map (
             CLK => CLK1, reset => reset, EN_4 => EN_4, EN_16 => EN_16, 
				 EN_32 => EN_32, EN_64 => EN_64, EN_128 => EN_128, 
				 EN_256 => EN_256, EN_4096 => EN_4096, EN_2048 => EN_2048, 
				 EN_1024 => EN_1024,EN_512 => EN_512, EN_16K => EN_16K, 
				 EN_65K => EN_65K, EN_1M => EN_1M);

-- 7 Segment Display shows DX1..DX4

 SEG0: Segcntrl port map (
             CLK1 => CLK1, Digit0 => DX1, Digit1 => DX2, 
				 Digit2 => DX3, Digit3 => DX4, 
             reset => reset, seg => seg, an => an, dp => dpx);

-- Output moving average of 8 buttonx BCode Multiplexer
   process(AVG, ACode,BCode) -- previous EN_16K
     begin
       if ( AVG = '1') then   
          DX4<= ACode(24 downto 21);
          DX3<= ACode(20 downto 17);
          DX2<= ACode(16 downto 13);
          DX1<= ACode(12 downto 9);
          DX0<= ACode(8 downto 5);
          DXA<= ACode(4 downto 1);
			 dp <='0';
       else
          DX4<= BCode(24 downto 21);
          DX3<= BCode(20 downto 17);
          DX2<= BCode(16 downto 13);
          DX1<= BCode(12 downto 9);
          DX0<= BCode(8 downto 5);
          DXA<= BCode(4 downto 1);
			 dp <='1';
       end if;
     end process;

-- Enable Averaging
   process(CLK1,EN_1M,reset,btn) -- previous EN_16K
     begin
       if ( reset = '1') then   
            AVG <='0';
       elsif  ((CLK1'event) and CLK1='1') and (EN_1M='1') and (btn(1)='1') then 
           AVG <= not(AVG);			   
       end if;
     end process;

-- Pipeline Control

	Inst_PipeControl: PipeControl PORT MAP(
		CLK => CLK1,
		EN => ENX,
		RESET => reset,
		Dual => sw(7), 
		MaxCount => Maxcount,
		Code => Code,
		VALID => Valid,
		JA => JAX,
		DIN => DIN 
	);
	  
   process(CLK1,Valid,reset,en_t) -- previous EN_16K
     begin
       if ( reset = '1') then   
            ACode<="0000000000000000000000000";
       elsif  ((CLK1'event) and CLK1='1') and EN_T='1' then 
		     -- moving average 255/256 ACode+ 1/256 BCode
           ACode <= unsigned(ACode) 
			           - unsigned( ACode( 24 downto 8))
						  + unsigned(BCode(24 downto 8));			   
       end if;
     end process;

-- Switches control data length 32, 16, 8, 2

		 MaxCount<= "111111111" when Sw(6 downto 5)="00" else 
                  "011111111" when Sw(6 downto 5)="01" else 
                  "001111111" when Sw(6 downto 5)="10" else 
                  "000011111";

-- Switch (4) control static or dynamic
		 JA<= JAX when Sw(4)='0' else 
            "010110";


-- Gray code decoding
 
   BCode(24) <= Code(24);
	BCode(23) <= not(Code(24)) xor Code(23);
	BCode(22) <= not(BCode(23)) xor Code(22);
	BCode(21) <= not(BCode(22)) xor Code(21);
	BCode(20) <= not(BCode(21)) xor Code(20);
	BCode(19) <= not(BCode(20)) xor Code(19);
	BCode(18) <= not(BCode(19)) xor Code(18);
	BCode(17) <= not(BCode(18)) xor Code(17);
	BCode(16) <= not(BCode(17)) xor Code(16);
	BCode(15) <= not(BCode(16)) xor Code(15);
	BCode(14) <= not(BCode(15)) xor Code(14);
	BCode(13) <= not(BCode(14)) xor Code(13);
	BCode(12) <= not(BCode(13)) xor Code(12);
	BCode(11) <= not(BCode(12)) xor Code(11);
	BCode(10) <= not(BCode(11)) xor Code(10);
	BCode(9) <= not(BCode(10)) xor Code(9);
	BCode(8) <= not(BCode(9)) xor Code(8);
	BCode(7) <= not(BCode(8)) xor Code(7);
	BCode(6) <= not(BCode(7)) xor Code(6);
	BCode(5) <= not(BCode(6)) xor Code(5);
	BCode(4) <= not(BCode(5)) xor Code(4);
	BCode(3) <= not(BCode(4)) xor Code(3);
	BCode(2) <= not(BCode(3)) xor Code(2);
	BCode(1) <= not(BCode(2)) xor Code(1);
	BCode(0) <= not(BCode(1)) xor Code(0);

-- Switches control CLK rate of sampling

       sel <= sw(3 downto 0);
		 ENX <= EN_1M when Sel="0000" else
            EN_65K when Sel="0001" else
            EN_16K when Sel="0010" else -- 32 times send
            EN_4096 when Sel="0011" else -- 8 times
				   -- EN_2048, EN_1024, EN_512
            EN_2048 when Sel="0100" else -- 4 times
            EN_1024 when Sel="0101" else -- 2 times
            EN_512 when Sel="0110" else -- 1 time
            EN_256 when Sel="0111" else -- buffer needed
            EN_128 when Sel="1000" else -- buffer needed
            EN_64 when Sel="1001" else -- buffer needed
            EN_32 when Sel="1010" else -- buffer needed
            EN_16 when Sel="1011" else -- buffer needed
             EN_65K;

		 EN_T <= EN_1M when Sel="0000" else  -- 8M x16 needed
            EN_65K when Sel="0001" else
            EN_65K when Sel="0010" else -- EN_65K x16 4 times send (x32 x8 send)
            EN_16K when Sel="0011" else -- 2 times send
				   -- EN_2048, EN_1024, EN_512
            EN_16K when Sel="0100" else -- 
            EN_16K when Sel="0101" else
            EN_16K when Sel="0110" else
            EN_16K when Sel="0111" else
            EN_16K when Sel="0111" else
            EN_16K when Sel="1000" else
            EN_16K when Sel="1001" else
            EN_16K when Sel="1010" else
            EN_16K when Sel="1011" else
            EN_65K;
-----------------------------------------------------------------------------------
-- RAM to buffer values
-- BCode and BRCode should have the same length as RAM address
-----------------------------------------------------------------------------------
	Inst_RAM: DPRAM 
	generic map(
      B => 16, -- number of bits
      W => WBUFFER -- number of address bits 2048 values
   )
	PORT MAP(
		clk => CLK1 ,
		we => we,
		w_data => BCode(24 downto 9) ,
		w_addr => countx(16 downto 3),
		r_addr => countx(16 downto 3),
		r_data => TCode(24 downto 9)
	);
   TCode(8 downto 0) <= "000000000";
	
-----------------------------------------------------------------------------------
-- Control RAM, UART
-- Fill RAM first, send via UART
-----------------------------------------------------------------------------------

   process(CLK1,reset) -- previous EN_16K
     begin
       if ( reset = '1') then   
		   --             F       7      0
         countx <="000000000000000000"; -- 16 k RAM * 8 + read/write = 18 bit 
			start <= '0';	
			wr_uart <= '0';
         we <= '0';
       elsif  ((CLK1'event) and CLK1='1')  then --  
			  if ( countx(16 downto 0) = "00000000000000000") then
			    start <= '1';
           else
			    start <= '0';
           end if;			  
			  wr_uart <= '0';
           we <= '0';			  
			  --------------------------------------------------------
			  --- Countx(17) works for 16k blocks, debugging countx(4)
           if ((tx_empty='1') and (Countx(17) ='1') ) then -- send to uart  
					 wr_uart <= not(wr_uart);
			  elsif ( (valid ='1') and (Countx(17) ='0')) then -- write RAM
				    we <= '1';
 			  end if;		 
           if  (wr_uart = '1')  then
	 				 countx <= unsigned(countx) + 1;					
			  elsif (we = '1') then
	 				 countx <= unsigned(countx) + 8;					
           end if;             					
       end if;
     end process;

-----------------------------------------------------------------------------------
-- UART implementation
-- Change EN_16K according to sampling rate
-- Counter to send 8 data  x.....<cr><lf>  
-----------------------------------------------------------------------------------
  
	-- send data xFFFFF<cr><lf>			
  decode <= countx(2 downto 0);
  i_data <= '0' & TCode(8 downto 5) when decode="101" else       -- former BCode
            '0' & TCode(12 downto 9) when decode="100" else
				'0' & TCode(16 downto 13) when decode="011" else
				'0' & TCode(20 downto 17) when decode="010" else
				'0' & TCode(24 downto 21) when decode="001" else
			--	"10000" when decode="001" else -- ASCII 0
				'1' & start & "001" when decode="000" else -- ASCII x   
				"10010" when decode="110" else -- ASCII cr
				"10011" when decode="111" else -- ASCII lf
				"10100";

  w_data <= "00110000" when i_data="00000" else   -- HEX 30 ASCII "0"
            "00110001" when i_data="00001" else   -- 1
            "00110010" when i_data="00010" else   -- 2
            "00110011" when i_data="00011" else   -- 3
            "00110100" when i_data="00100" else   -- 4
            "00110101" when i_data="00101" else   -- 5
            "00110110" when i_data="00110" else   -- 6
            "00110111" when i_data="00111" else   -- 7
            "00111000" when i_data="01000" else   -- 8
            "00111001" when i_data="01001" else   -- 9
            "01000001" when i_data="01010" else   -- A
            "01000010" when i_data="01011" else   -- b
            "01000011" when i_data="01100" else   -- c
            "01000100" when i_data="01101" else   -- d
            "01000101" when i_data="01110" else   -- e
            "01000110" when i_data="01111" else   -- f
            "00110000" when i_data="10000" else  -- 0  
            "01111000" when i_data="10001" else  -- x
				"00001101" when i_data="10010" else  -- cr
				"00001010" when i_data="10011" else  -- lf
				"00100000" when i_data="10100" else  -- space
				"00100000";

  UART0: uart   port map(
      clk=>CLK1, reset=>reset, rd_uart=>'0', wr_uart=>wr_uart, rx=>RXD,
      w_data => w_data, tx_full=> tx_full, tx_empty => tx_empty, rx_empty=> rx_empty,
      r_data => RD_data, tx=> TXI
   );
   -- Monitor seriell Interface
	-- Baud Rate 38400 8 Bit N -1  
	TXD <= TXI;
	JB(0) <= TXI;
	JB(1) <= DIN(1);
	JB(2) <= DIN(0);
	JB(3) <= Valid;
	
end Behavioral;

